-- ===========================================================================
-- ===========================================================================
--    U N I Q U E    T A B L E   C R E A T I O N
-- ===========================================================================
-- ===========================================================================


--=|Make the Unique Member Signatures table.|=--

    DROP TABLE IF EXISTS unique_member_signatures;
    DROP INDEX IF EXISTS unique_member_signatures__name;
    DROP INDEX IF EXISTS unique_member_signatures__arguments;
    DROP INDEX IF EXISTS unique_member_signatures__member_type;
  
    
    CREATE TABLE unique_member_signatures 
    AS
    SELECT   wk_members.name                AS name,
             wk_members.arguments           AS arguments,
	           wk_members.wk_member_types_fk  AS member_type,
	           count(id)                      AS member_count
    
    FROM     wk_members
    
    GROUP BY name,
             arguments,
          	 member_type
    
    ORDER BY name,
             arguments,
          	 member_type
;


    CREATE INDEX unique_member_signatures__name
              ON unique_member_signatures( name )
;
    CREATE INDEX unique_member_signatures__arguments
              ON unique_member_signatures( arguments )
;
    CREATE INDEX unique_member_signatures__member_type
              ON unique_member_signatures( member_type )
;
--=|Make the Unique Member Names table.|=--

    DROP TABLE IF EXISTS unique_member_names;
    DROP INDEX IF EXISTS unique_member_names__name;

    
    CREATE TABLE unique_member_names
    AS
    SELECT   name              AS name,
	           count(name)       AS signature_count,
	           sum(member_count) AS member_count
    
    FROM     unique_member_signatures
    
    GROUP BY name
    ORDER BY name
;


    CREATE INDEX unique_member_names__name
              ON unique_member_names( name )
;
--=|Make the Unique Signatures table.|=--

    DROP TABLE IF EXISTS unique_signatures;
    DROP INDEX IF EXISTS unique_signatures__arguments;


    CREATE TABLE unique_signatures
    AS
    SELECT                                arguments,
	                                        member_type,

	           count( member_type )      AS name_count,
	           sum( member_count )       AS member_count
    
    FROM     unique_member_signatures
    
    GROUP BY arguments, member_type

    ORDER BY arguments, member_type

;
    CREATE INDEX unique_signatures__arguments
              ON unique_signatures( arguments )
;

--=|Make the Unique Arguments table.|=--

    DROP TABLE IF EXISTS unique_arguments;
    DROP INDEX IF EXISTS unique_arguments__arguments;


    CREATE TABLE unique_arguments
    AS
    SELECT   arguments                 AS arguments,
	           count( member_type )      AS member_type_count,
	           sum( name_count )         AS name_count,
	           sum( member_count )       AS member_count
    
    FROM     unique_signatures
    
    GROUP BY arguments
    ORDER BY arguments
;

    CREATE INDEX unique_arguments__arguments
              ON unique_arguments( arguments )
;



--=|Make the Unique Class Names table.|=--

    DROP TABLE IF EXISTS unique_class_names;
    DROP INDEX IF EXISTS unique_class_names__class_name;

    CREATE TABLE unique_class_names
    AS

    SELECT     wk_classes.name                        AS class_name,
               count( wk_classes.wk_components_fk )   AS component_count

    FROM       wk_classes
            
    GROUP BY   class_name

    ORDER BY   class_name
;

    CREATE INDEX unique_class_names__class_name
              ON unique_class_names( class_name )
;

--=|Make the Unique Class Members table.|=--

    DROP TABLE IF EXISTS unique_class_members;
    DROP INDEX IF EXISTS unique_class_members__class_name;
    DROP INDEX IF EXISTS unique_class_members__member_name;

    CREATE TABLE unique_class_members
    AS

    SELECT     wk_classes.name                        AS class_name,
               wk_members.name                        AS member_name,
               count( wk_members.id )                 AS count

    FROM       wk_members
    INNER JOIN                            wk_classes
    ON         wk_members.wk_classes_fk = wk_classes.id
            
    GROUP BY   class_name, member_name

    ORDER BY   class_name, member_name
;
    CREATE INDEX unique_class_members__class_name
              ON unique_class_members( class_name )
;
    CREATE INDEX unique_class_members__member_name
              ON unique_class_members( member_name )
;

--=|Make the Unique Keywords table.|=--

    DROP TABLE IF EXISTS unique_keywords;
    DROP INDEX IF EXISTS unique_keywords__keyword;


    CREATE TABLE unique_keywords
    AS

    SELECT     wk_members.name                        AS keyword

    FROM       wk_members
    INNER JOIN                            wk_classes
    ON         wk_members.wk_classes_fk = wk_classes.id

    WHERE      wk_classes.name = "."
	
    GROUP BY   keyword

    ORDER BY   keyword
;

    CREATE INDEX unique_keywords__keyword
              ON unique_keywords( keyword )
;


